import {
  select,
  csv,
  scaleLinear,
  extent,
  axisLeft,
  schemeCategory10,
  scaleOrdinal,
  axisBottom
} from 'd3';
import { dropdownMenu } from './dropdownMenu';
import { scatterPlot } from './scatterPlot';
import { colorLegend } from './colorLegend';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

let data;
let xColumn;
let yColumn;
let colorColumn;

const onXColumnClicked = column => {
  xColumn = column;
  render();
};

const onYColumnClicked = column => {
  yColumn = column;
  render();
};

const onColorColumnClicked = column => {
  colorColumn = column;
  render();
};

const render = () => {
  
  const colorValue = d => d[colorColumn];
  
  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(schemeCategory10);
  
  
  select('#x-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onXColumnClicked,
      selectedOption: xColumn
    });
  
  select('#y-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onYColumnClicked,
      selectedOption: yColumn
    });
  
  select('#color-menu')
    .call(dropdownMenu, {
      options: data.columns,
      onOptionClicked: onColorColumnClicked,
      selectedOption: colorColumn
    });
  
  const circleRadius = 10;
  
  svg.call(scatterPlot, {
    xValue: d => d[xColumn],
    xAxisLabel: xColumn,
    yValue: d => d[yColumn],
    colorScale,
    colorValue,
    circleRadius,
    yAxisLabel: yColumn,
    margin: { top: 10, right: 40, bottom: 88, left: 150 },
    width,
    height,
    data
  });
  
  svg.append('g')
    .attr('transform', `translate(800,50)`)
    .call(colorLegend, {
      colorScale,
      circleRadius,
      spacing: 40,
      textOffset: 20
    });
};

csv('https://vizhub.com/curran/datasets/auto-mpg.csv')
  .then(loadedData => {
    data = loadedData;
    data.forEach(d => {
      d.mpg = +d.mpg;
      d.cylinders = +d.cylinders;
      d.displacement = +d.displacement;
      d.horsepower = +d.horsepower;
      d.weight = +d.weight;
      d.acceleration = +d.acceleration;
      d.year = +d.year;  
    });
    xColumn = data.columns[4];
    yColumn = data.columns[0];
    colorColumn = data.columns[1];
  
    render();
  });